@@ -18,10 +18,10 @@
 # under the License.
 
 # coding=utf-8
-import os
-import sys
 import httplib
 import warnings
+import urllib
+
 warnings.filterwarnings("once", category=DeprecationWarning)
 
 try:
@@ -36,6 +36,8 @@ from .acs_exception.exceptions import ServerException
 from .acs_exception import error_code, error_msg
 from .http.http_response import HttpResponse
 from .request import AcsRequest
+from .http import format_type
+from .auth.Signer import Signer
 
 """
 Acs default client module.
@@ -45,17 +47,25 @@ Created on 6/15/2015
 @author: alex jiang
 """
 
+DEFAULT_SDK_CONNECTION_TIMEOUT_IN_SECONDS = 10
+
 
 class AcsClient:
+
     def __init__(
             self,
-            ak,
-            secret,
-            region_id,
+            ak=None,
+            secret=None,
+            region_id="cn-hangzhou",
             auto_retry=True,
             max_retry_time=3,
             user_agent=None,
-            port=80):
+            port=80,
+            timeout=DEFAULT_SDK_CONNECTION_TIMEOUT_IN_SECONDS,
+            public_key_id=None,
+            private_key=None,
+            session_period=3600,
+            debug=False):
         """
         constructor for AcsClient
         :param ak: String, access key id
@@ -65,6 +75,7 @@ class AcsClient:
         :param max_retry_time: Number
         :return:
         """
+
         self.__max_retry_num = max_retry_time
         self.__auto_retry = auto_retry
         self.__ak = ak
@@ -72,9 +83,19 @@ class AcsClient:
         self.__region_id = region_id
         self.__user_agent = user_agent
         self._port = port
-        self._location_service = LocationService(self)
+        self._location_service = LocationService(self, timeout=timeout)
+        self._timeout = timeout
         # if true, do_action() will throw a ClientException that contains URL
         self._url_test_flag = False
+        credential = {
+            'ak': ak,
+            'secret': secret,
+            'public_key_id': public_key_id,
+            'private_key': private_key,
+            'session_period': session_period,
+            'region_id': region_id
+        }
+        self._signer = Signer.get_signer(credential, debug)
 
     def get_region_id(self):
         """
@@ -117,12 +138,6 @@ class AcsClient:
     def set_region_id(self, region):
         self.__region_id = region
 
-    def set_access_key(self, ak):
-        self.__ak = ak
-
-    def set_access_secret(self, secret):
-        self.__secret = secret
-
     def set_max_retry_num(self, num):
         """
         set auto retry number
@@ -147,17 +162,18 @@ class AcsClient:
         """
         self.__user_agent = agent
 
-    def get_location_service(self):
-        return self._location_service
-
     def get_port(self):
         return self._port
 
+    def get_location_service(self):
+        return self._location_service
+
     def _resolve_endpoint(self, request):
         endpoint = None
         if request.get_location_service_code() is not None:
             endpoint = self._location_service.find_product_domain(
-                self.get_region_id(), request.get_location_service_code())
+                self.get_region_id(), request.get_location_service_code(), request.get_product(),
+                request.get_location_endpoint_type())
         if endpoint is None:
             endpoint = region_provider.find_product_domain(
                 self.get_region_id(), request.get_product())
@@ -172,31 +188,35 @@ class AcsClient:
         return endpoint
 
     def _make_http_response(self, endpoint, request):
-        content = request.get_content()
+        body_params = request.get_body_params()
+        if body_params:
+            body = urllib.urlencode(body_params)
+            request.set_content(body)
+            request.set_content_type(format_type.APPLICATION_FORM)
+        elif request.get_content():
+            request.set_content_type(format_type.APPLICATION_OCTET_STREAM)
         method = request.get_method()
-        header = request.get_signed_header(
-            self.get_region_id(),
-            self.get_access_key(),
-            self.get_access_secret())
+        header, url = self._signer.sign(self.__region_id, request)
+
         if self.get_user_agent() is not None:
             header['User-Agent'] = self.get_user_agent()
-            header['x-sdk-client'] = 'python/2.0.0'
         if header is None:
             header = {}
+        header['x-sdk-client'] = 'python/2.0.0'
 
         protocol = request.get_protocol_type()
-        url = request.get_url(
-            self.get_region_id(),
-            self.get_access_key(),
-            self.get_access_secret())
         response = HttpResponse(
             endpoint,
             url,
             method,
             header,
             protocol,
-            content,
-            self._port)
+            request.get_content(),
+            self._port,
+            timeout=self._timeout)
+        if body_params:
+            body = urllib.urlencode(request.get_body_params())
+            response.set_content(body, "utf-8", format_type.APPLICATION_FORM)
         return response
 
     def _implementation_of_do_action(self, request):
@@ -275,3 +295,4 @@ class AcsClient:
             "get_response() method is deprecated, please use do_action_with_exception() instead.",
             DeprecationWarning)
         return self._implementation_of_do_action(acs_request)
+
