@@ -25,13 +25,14 @@ import sys
 parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 sys.path.insert(0, parent_dir)
 
-from .http import protocol_type as PT
-from .http import method_type as MT
-from .http import format_type as FT
-from .auth import rpc_signature_composer as rpc_signer
-from .auth import roa_signature_composer as roa_signer
-from .auth import oss_signature_composer as oss_signer
-from .auth import md5_tool
+from .http import protocol_type as pt
+from .http import method_type as mt
+from .http import format_type as ft
+from .auth.composer import rpc_signature_composer as rpc_signer
+from .auth.composer import roa_signature_composer as roa_signer
+from .auth.composer import oss_signature_composer as oss_signer
+from .auth.utils import md5_tool
+from aliyunsdkcore.auth.algorithm import sha_hmac1
 import abc
 import base64
 
@@ -57,8 +58,9 @@ class AcsRequest:
     def __init__(self, product, version=None,
                  action_name=None,
                  location_service_code=None,
+                 location_endpoint_type='openAPI',
                  accept_format=None,
-                 protocol_type=PT.HTTP,
+                 protocol_type=pt.HTTP,
                  method=None):
         """
 
@@ -79,22 +81,25 @@ class AcsRequest:
         self.__params = {}
         self.__method = method
         self.__header = {}
-        self.__domain_params = None
-        self.__domain_pattern = None
+        self.__body_params = {}
         self.__uri_pattern = None
         self.__uri_params = None
         self.__content = None
         self.__location_service_code = location_service_code
+        self.__location_endpoint_type = location_endpoint_type
 
     def add_query_param(self, k, v):
         if self.__params is None:
             self.__params = {}
         self.__params[k] = v
 
-    def add_domain_param(self, k, v):
-        if self.__domain_params is None:
-            self.__domain_params = {}
-        self.__domain_params[k] = v
+    def add_body_params(self, k, v):
+        if self.__body_params is None:
+            self.__body_params = {}
+        self.__body_params[k] = v
+
+    def get_body_params(self):
+        return self.__body_params
 
     def get_uri_pattern(self):
         return self.__uri_pattern
@@ -102,12 +107,6 @@ class AcsRequest:
     def get_uri_params(self):
         return self.__uri_params
 
-    def get_domain_pattern(self):
-        return self.__domain_pattern
-
-    def get_domain_params(self):
-        return self.__domain_params
-
     def get_product(self):
         return self.__product
 
@@ -144,9 +143,6 @@ class AcsRequest:
     def set_version(self, version):
         self.__version = version
 
-    def set_domain_param(self, param):
-        self.__domain_params = param
-
     def set_action_name(self, action_name):
         self.__action_name = action_name
 
@@ -204,6 +200,15 @@ class AcsRequest:
     def get_location_service_code(self):
         return self.__location_service_code
 
+    def get_location_endpoint_type(self):
+        return self.__location_endpoint_type
+
+    def set_content_type(self, content_type):
+        self.add_header("Content-Type", content_type)
+
+    def get_location_endpoint_type(self):
+        return self.__location_endpoint_type
+
     @abc.abstractmethod
     def get_style(self):
         pass
@@ -222,9 +227,28 @@ class RpcRequest(AcsRequest):
     Class to compose an RPC style request with.
     """
 
-    def __init__(self, product, version, action_name, location_service_code=None, format=None, protocol=None):
-        AcsRequest.__init__(self, product, version, action_name, location_service_code, format, protocol, MT.GET)
+    def __init__(
+            self,
+            product,
+            version,
+            action_name,
+            location_service_code=None,
+            location_endpoint_type='openAPI',
+            format=None,
+            protocol=None,
+            signer=sha_hmac1):
+        AcsRequest.__init__(
+            self,
+            product,
+            version,
+            action_name,
+            location_service_code,
+            location_endpoint_type,
+            format,
+            protocol,
+            mt.GET)
         self.__style = STYLE_RPC
+        self.__signer = signer
 
     def get_style(self):
         return self.__style
@@ -236,13 +260,21 @@ class RpcRequest(AcsRequest):
         req_params['Version'] = self.get_version()
         req_params['Action'] = self.get_action_name()
         req_params['Format'] = self.get_accept_format()
+
         return req_params
 
     def get_url(self, region_id, ak, secret):
         sign_params = self.__get_sign_params()
         if 'RegionId' not in sign_params.keys():
             sign_params['RegionId'] = region_id
-        url = rpc_signer.get_signed_url(sign_params, ak, secret, self.get_accept_format(), self.get_method())
+        url = rpc_signer.get_signed_url(
+            sign_params,
+            ak,
+            secret,
+            self.get_accept_format(),
+            self.get_method(),
+            self.get_body_params(),
+            self.__signer)
         return url
 
     def get_signed_header(self, region_id=None, ak=None, secret=None):
@@ -254,8 +286,18 @@ class RoaRequest(AcsRequest):
     Class to compose an ROA style request with.
     """
 
-    def __init__(self, product, version, action_name, location_service_code=None, method=None, headers=None, uri_pattern=None, path_params=None,
-                 protocol=None):
+    def __init__(
+            self,
+            product,
+            version,
+            action_name,
+            location_service_code=None,
+            location_endpoint_type='openAPI',
+            method=None,
+            headers=None,
+            uri_pattern=None,
+            path_params=None,
+            protocol=None):
         """
 
         :param product: String, mandatory
@@ -268,7 +310,16 @@ class RoaRequest(AcsRequest):
         :param protocol: String
         :return:
         """
-        AcsRequest.__init__(self, product, version, action_name, location_service_code, FT.RAW, protocol, method)
+        AcsRequest.__init__(
+            self,
+            product,
+            version,
+            action_name,
+            location_service_code,
+            location_endpoint_type,
+            ft.RAW,
+            protocol,
+            method)
         self.__style = STYLE_ROA
         self.__method = method
         self.__header = headers
@@ -310,18 +361,21 @@ class RoaRequest(AcsRequest):
         :param secret: String
         :return: Dict
         """
-        sign_params = self.get_query_params()
-        if (self.get_content() is not None):
+        sign_params = self.__get_sign_params()
+        if self.get_content() is not None:
             md5_str = md5_tool.get_md5_base64_str(self.get_content())
             self.add_header('Content-MD5', md5_str)
         if 'RegionId' not in sign_params.keys():
             sign_params['RegionId'] = region_id
-        signed_headers = roa_signer.get_signature_headers(sign_params, ak, secret,
-                                                          self.get_accept_format(),
-                                                          self.get_headers(),
-                                                          self.get_uri_pattern(),
-                                                          self.get_path_params(),
-                                                          self.get_method())
+        signed_headers = roa_signer.get_signature_headers(
+            sign_params,
+            ak,
+            secret,
+            self.get_accept_format(),
+            self.get_headers(),
+            self.get_uri_pattern(),
+            self.get_path_params(),
+            self.get_method())
         return signed_headers
 
     def get_url(self, region_id, ak=None, secret=None):
@@ -333,13 +387,26 @@ class RoaRequest(AcsRequest):
         sign_params = self.get_query_params()
         if region_id not in sign_params.keys():
             sign_params['RegionId'] = region_id
-        url = roa_signer.get_url(self.get_uri_pattern(), sign_params, self.get_path_params())
+        url = roa_signer.get_url(
+            self.get_uri_pattern(),
+            sign_params,
+            self.get_path_params())
         return url
 
 
 class OssRequest(AcsRequest):
-    def __init__(self, product, version, action_name, location_service_code, bucket=None, method=None,
-                 headers=None, uri_pattern=None, path_params=None, protocol=None):
+    def __init__(
+            self,
+            product,
+            version,
+            action_name,
+            location_service_code,
+            bucket=None,
+            method=None,
+            headers=None,
+            uri_pattern=None,
+            path_params=None,
+            protocol=None):
         """
 
         :param product: String, mandatory
@@ -353,7 +420,16 @@ class OssRequest(AcsRequest):
         :param protocol: String
         :return:
         """
-        AcsRequest.__init__(self, product, version, action_name, location_service_code, FT.XML, protocol, method)
+        AcsRequest.__init__(
+            self,
+            product,
+            version,
+            action_name,
+            location_service_code,
+            'openAPI',
+            ft.XML,
+            protocol,
+            method)
         self.__style = STYLE_OSS
         self.__bucket = bucket
         self.__method = method
@@ -399,10 +475,16 @@ class OssRequest(AcsRequest):
         sign_params = self.get_query_params()
         if 'RegionId' not in sign_params.keys():
             sign_params['RegionId'] = region_id
-        signed_headers = oss_signer.get_signature_headers(sign_params, ak, secret, self.get_accept_format(),
-                                                          self.get_headers(),
-                                                          self.get_uri_pattern(), self.get_path_params(),
-                                                          self.get_method(), self.__bucket)
+        signed_headers = oss_signer.get_signature_headers(
+            sign_params,
+            ak,
+            secret,
+            self.get_accept_format(),
+            self.get_headers(),
+            self.get_uri_pattern(),
+            self.get_path_params(),
+            self.get_method(),
+            self.__bucket)
         return signed_headers
 
     def get_url(self, region_id, ak=None, secret=None):
@@ -414,5 +496,8 @@ class OssRequest(AcsRequest):
         sign_params = self.get_query_params()
         if 'RegionId' not in sign_params.keys():
             sign_params['RegionId'] = region_id
-        url = oss_signer.get_url(sign_params, self.get_uri_pattern(), self.get_path_params())
+        url = oss_signer.get_url(
+            sign_params,
+            self.get_uri_pattern(),
+            self.get_path_params())
         return url
